from rest_framework.generics import GenericAPIView
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from .models import Privatbank
from .serializers import (
    PrivatTokenSerializer,
    PaymentSerializer,
    PrivatPeriodSerializer,
)
from .exceptions import (
    PrivatBankDoesNotExistsException,
    PrivatBankExistsException,
)

from drf_privatbank.managers import PrivatManager


class PrivatBankView(GenericAPIView):
    serializer_class = PrivatTokenSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        _ = serializer.validated_data
        privat_obj = Privatbank.objects.filter(user=self.request.user)
        if privat_obj.first() is not None:
            raise PrivatBankExistsException
        privat_obj.create(
            privat_token=_["privat_token"],
            iban_UAH=_["iban_UAH"],
            user=request.user
        )
        return Response(
            {"detail": "Privatbank credentials added successfully."},
            status.HTTP_201_CREATED
        )
    
    def put(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        _ = serializer.validated_data
        privat_obj = Privatbank.objects.filter(user=self.request.user)
        if privat_obj.first() is not None:
            privat_obj.update(
                privat_token=_["privat_token"],
                iban_UAH=_["iban_UAH"]
            )
            return Response(
                {"detail": "Privatbank credentials changed successfully."}
            )
        
        raise PrivatBankDoesNotExistsException
    
    def delete(self, request):
        privat_obj = Privatbank.objects.filter(user=self.request.user)
        if privat_obj.first() is not None:
            privat_obj.delete()
            return Response(status.HTTP_204_NO_CONTENT)
        raise PrivatBankDoesNotExistsException


class PrivatBankCurrencyCashRate(APIView):
    
    def get(self, request):
        mng = PrivatManager(request)
        payload = mng.get_currency(cashe_rate=True)
        return Response(payload)


class PrivatBankCurrencyNonCashRate(APIView):

    def get(self, request):
        mng = PrivatManager(request)
        payload = mng.get_currency(cashe_rate=False)
        return Response(payload)


class PrivatBankInfo(APIView):

    def get(self, request):
        privat_obj = Privatbank.objects.filter(user=self.request.user).first()
        if privat_obj is not None:
            mng = PrivatManager(request)
            mng.token = privat_obj.privat_token
            mng.iban = privat_obj.iban_UAH
            payload = mng.get_client_info()
            return Response(payload)
        raise PrivatBankDoesNotExistsException


class BalancePrivatView(APIView):

    def get(self, request):
        privat_obj = Privatbank.objects.filter(user=self.request.user).first()
        if privat_obj is not None:
            mng = PrivatManager(request)
            mng.token = privat_obj.privat_token
            mng.iban = privat_obj.iban_UAH
            payload = mng.get_balance()
            return Response(payload)
        raise PrivatBankDoesNotExistsException


class StatementPrivatView(GenericAPIView):
    serializer_class = PrivatPeriodSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        _ = serializer.validated_data
        privat_obj = Privatbank.objects.filter(user=self.request.user).first()
        if privat_obj is not None:
            mng = PrivatManager(request)
            mng.token = privat_obj.privat_token
            mng.iban = privat_obj.iban_UAH
            period = _["period"]
            limit = _["limit"]
            payload = mng.get_statement(period, limit)
            return Response(payload)
        raise PrivatBankDoesNotExistsException


class PaymentPrivatView(GenericAPIView):
    serializer_class = PaymentSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        _ = serializer.validated_data
        privat_obj = Privatbank.objects.filter(user=self.request.user).first()
        if privat_obj is not None:
            mng = PrivatManager(request)
            mng.token = privat_obj.privat_token
            mng.iban = privat_obj.iban_UAH
            payload = mng.create_payment(_["recipient"], str(_["amount"]))
            return Response(payload)
        raise PrivatBankDoesNotExistsException
